function Fout_ch = Channel ( Fin, Fin_tx)
% Version 1.2
% Last Modified: May 24th, 2009
% Ali Tawfiq
% Multimedia Labratory, The Edward S. Rogers Sr. Dept of Electrical and
% Computer Engineering, University of Toronto

%**************************************************************************
%       Fin     -> L: Length of signal
%               -> itt: number of iterations
%               -> Mod: Modulation Type (refer to 'Mapping.m' for allowed
%                        types)
%               -> M: M-ary, binary (2), quade (4) or 2^n
%               -> k: Number of bits per symbol
%               -> Block: String for block type (refer to 'Mapping.m')
%               -> BlockL: Length of blocks (refer to 'Mapping.m')
%               -> Numtx: Number of transmit antennas
%               -> Numrx: Number of receive antennas
%               -> SNR: a vector of SNR values
%               -> sBER: a temporary vector to store the computed BER
%               -> BER: a zero vector to store the computed BER
%               -> Out: a zero vector to store the final received signal
%                       to be be saved for reference
%               -> InSig: The input signal in bits
%               -> S: Numtx column vectors representing the split input
%                      signal
%               -> Saving: 'backup' for saving the data. 
%               -> InSym: Numtx column vectors of the modulated signal.
%               -> Px: Power of signal (1 for all psk, diff for QAM)
%               Filter Parameters:
%               -> r: Roll-off-factor Used in raised cosine filter
%               -> T: Bit period
%               -> dur:  Duration
%               -> FltrType: The type of filter generated. 
%               -> nsamp: oversampling rate
%               -> Delay: The delay experienced in the filter
%               -> fltr: The generated filter (To be applied in Transmit.m
%                        and Receiver.m)
%       Fin_tx  -> tx[1:Numtx] = data transmitted from each antenna
%
%       Parameters added in this module:
%       Fout_ch -> Ch = array with channel coeficients as column vectors
%               -> R = array with received signals as the row vectors
%**************************************************************************
addpath ./Channel
%% ==================== Channel ========================== 
%%========================================================
    % Create the channel Coeficients
    for uu = 1: Fin.Numtx * Fin.Numrx
        Fout_ch.Ch(:,uu)= Rayleigh(Fin.nsamp,length(Fin_tx.tx)/Fin.nsamp,Fin.T);
        %Or user might also use the IFFT method implemented in 'ricefad.m'
        %instead
            %Fout_ch.Ch(:,uu)= ricefad(length(Fin_tx.tx0),Fin.fdt);
    end
 
    % Calculate the energy of the filter
    Ef= sum(abs(Fin.fltr));
    % Generate Random Noise
    for o = 1: length (Fin.SNR)
        Pn= (Fin.Px*Ef) / (10^(Fin.SNR(o)/10));
        for oo= 1: Fin.Numrx
            temp= sqrt(Pn/2) * (randn(length(Fin_tx.tx)*10,1) + i*randn(length(Fin_tx.tx)*10,1));
            randomIndex= randperm(length(temp));
            temp= temp(randomIndex);
            n(:,oo)= temp(1:10:length(temp));
        end
        
        % Formulate the Received Signals. As follows:
        % R1= Ch1*Tx1          + Ch2*Tx2         + ... + Ch(Numtx)*Tx(Numtx)
        % R2= Ch1(Numtx+1)*Tx1 + Ch(Numtx+2)*Tx2 + ... + Ch(Numtx*2)*Tx(Numtx*2)
        % ...
        % R(Numrx)= .....
        count=1; % Temporary Counter
            for kk=1:Fin.Numrx
                % Create a zero vector
                Fout_ch.R(o,:,kk)= zeros(1,length(Fin_tx.tx),1);
                for jj=1:Fin.Numtx
                    % Summation of 'Numtx' times 
                    Fout_ch.R(o,:,kk) = Fout_ch.R(o,:,kk).' + Fout_ch.Ch(:,count,1) .* Fin_tx.tx(:,jj,1);
                    count=count+1; % Increment counter
                end
                % Adding noise to received Signal
                Fout_ch.R(o,:,kk)= Fout_ch.R(o,:,kk) + n(:,kk,1).';
            end
    end
    
