function x=ricefad(nopoints,fm,sigma,s)

% Modified: Feb 20th, 2009
% Edited by Ali Tawfiq
% Multimedia Labratory, The Edward S. Rogers Sr. Dept of Electrical and
% Computer Engineering, University of Toronto

%**************************************************************************
% This function is not written by me (Ali Tawfiq), Credit will be given
% once i know the publisher. 
% 
% Function returns (nopoints) complex Gaussian samples with
% possibly nonzero mean.  The square of the parameter s is the
% equal to the square of the mean of the real part plus the square 
% of the mean of the imaginary part, and the 
% autocovariance function of the real or imaginary part approximates 
% sigma^2*bessel(0,2*pi*fm*d), where d is the sample lag and 
% fm is the maximum Doppler frequency normalized by the sample rate.
%
% The elements of r=abs(x) have a Ricean distribution, 
% p(r)=(r/sigma^2)*exp(-(r^2+s^2)/(2*sigma^2))*besseli(0,r*s/(sigma^2)),
% r>=0, where sigma^2 is the variance of the each Gaussian component.
%
% If s=0, the elements of r=abs(x) have a Rayleigh distribution,
% p(r)=(1/2*(sigma^2))*exp(-r/(2*(sigma^2)))
% 
% The default value for sigma is sqrt(0.5); i.e. real and imaginary
% parts each have variance of 0.5.
% The default value for s is zero (i.e., Rayleigh fading)
%
% fm should be in the range 0<fm<0.5; i.e. the sample rate should
% be at least twice the maximum fading bandwidth.
%
% Note that randn() is not seeded within the routine.
%
% Reference:  D.J. Young and N.C. Beaulieu, "The Generation of
% Correlated Rayleigh Random Variates by Discrete Fourier Transform", 
% IEEE Transactions on Communications, vol. 48, pp. 1114-1227, July 2000.
%**************************************************************************

% Check for number of input arguments and assign defaults
if (nargin == 2)
  s = 0;
  sigmas = 0.5;
elseif (nargin == 3)
  s = 0;
  if (sigma<=0)
    error('Error: Sigma should be greater than zero.');
  end;
  sigmas = sigma^2;
elseif (nargin == 4)
  if (sigma<=0)
    error('Error: Sigma should be greater than zero.');
  end;
  sigmas = sigma^2;
  if (s<0) 
    error('Error: Non-centrality parameter should be non-negative.');
  end;
else
  error('Invalid number of input arguments - check usage.')
end;

% Check for range of fm
if ((fm >= 0.5) | (fm <=0)) 
  error('Error:  fm should be in the range 0 < fm < 0.5');
end;

% Check for sensible nopoints
l = log2(nopoints);
%if (round(l)~=l) 
%  warning(['Note:  This routine (FFT-based) runs faster with nopoints a power of two.'])
%end;

  
% Find number of nonzero frequency coefficients
fdn = nopoints*fm;
im=floor(fdn);

% Form filter
f = zeros(1,nopoints);
t = 0:im-1;
%f = 1 ./ sqrt( sqrt( 1.0 - (t./fdn).^2));
f = ( 1.0 - (t./fdn).^2).^(-.25);
f(im+1) = sqrt(im * (pi/2 - atan((im-1.0)/sqrt(2*im-1))));
f(1) = 0;

% Normalize filter so variance of real and imaginary output is 0.5
f = f * sqrt(sigmas*(nopoints^2) / (2.0 * sum(f.^2)));

% Multiply filter by N(0,1) random numbers
x = [(f.*randn(1,im+1) - j*f.*randn(1,im+1)) zeros(1,nopoints-2*im-1) f((im+1):-1:2).*randn(1,im) - j*f((im+1):-1:2).*randn(1,im)];

% Set value of first element for Ricean noncentrality (zero for Rayleigh)

x(1) = nopoints*s;

% Take inverse FFT
x = ifft(x);

% Complex Gaussian process is returned in complex matrix x.
% For Rayleigh envelope output, uncomment the following:
x = abs(x);

% plot(x)
% title('Correlated Rayleigh Envelope')
% xlabel('points')
% ylabel('Envelope')
