function Fout = CreateSig 
% Version 1.2
% Last Modified: May 24th, 2009
% Ali Tawfiq
% Multimedia Labratory, The Edward S. Rogers Sr. Dept of Electrical and
% Computer Engineering, University of Toronto

%**************************************************************************
% This is Module sets the different variables used for the specified
% communication system. It also generates the input signal (Or used can
% input their own signal).
%
%       Fout    -> L: Length of signal
%               -> itt: number of iterations
%               -> Mod: Modulation Type (refer to 'Mapping.m' for allowed
%                        types)
%               -> M: M-ary, binary (2), quade (4) or 2^n
%               -> k: Number of bits per symbol
%               -> Block: String for block type (refer to 'Mapping.m')
%               -> BlockL: Length of blocks (refer to 'Mapping.m')
%               -> Numtx: Number of transmit antennas
%               -> Numrx: Number of receive antennas
%               -> SNR: a vector of SNR values
%               -> sBER: a temporary vector to store the computed BER
%               -> BER: a zero vector to store the computed BER
%               -> Out: a zero vector to store the final received signal
%                       to be be saved for reference
%               -> InSig: The input signal in bits
%               -> S: Numtx column vectors representing the split input
%                      signal
%               -> Saving: 'backup' for saving the data. 
%               Filter Parameters:
%               -> r: Roll-off-factor Used in raised cosine filter
%               -> T: Bit period
%               -> dur:  Duration
%               -> FltrType: The type of filter generated. 
%               -> nsamp: oversampling rate
%**************************************************************************


addpath ./Modulation
%% ================== Setting Variables ==================
%%========================================================
% Setting General Parameters. 
Fout.L = 5000; % The length of the input signal
Fout.itt= 500; % the number of itterations
Fout.Mod = 'BPSK'; %Type of mapping
Fout.M = 2; 
Fout.K = log2(Fout.M); % Number of bits per symbole
Fout.Block= '0 Symbol';
Fout.BlockL= 10;
Fout.Numtx = 2 ; % Number of tranmist antennas
Fout.Numrx = 2 ; % Number of receive antennas
Fout.SNR= [0:14];
Fout.sBER= zeros (1, length(Fout.SNR));
Fout.BER= zeros (1, length(Fout.SNR));
Fout.Out=0;%Vector that will hold the final outputs
Fout.Saving = 'backup';

% Setting the Filter parameters
Fout.r = 0.25 ;%roll-off
Fout.T = 0.01; % Bit period
Fout.dur =  0.05; %Duration
Fout.FltrType= 'sqrtrcos';% Type of Filter
Fout.nsamp = 2^2; % Oversampling rate 

%% ================== Binary Signal ======================
%%========================================================
% create random signal of bits
% If user wishes to input their own signal they should replace this below.
% Note that L and itt must be changed accordingly if user inputs their own
% signal. l*itt*K= Signal Length. 
Fout.InSig= (binary((Fout.L)*(Fout.K)*(Fout.itt))).';

%% ================== Splitting signal ===================
%%========================================================
% here we split the signal generated above accordingly into [Numtx] Number
% of signals. If Numtx=2 and Insig=X1X2X3X4... , then we get S1=X1X3X5...
% and S2=X2X4X6...
for kk= 1:Fout.Numtx
    Fout.S(:,kk)= Fout.InSig(kk:Fout.Numtx:length(Fout.InSig));
end
