function Fout = Mapping(Fin)
% Version 1.2
% Last Modified: May 24th, 2009
% Modified by Ali Tawfiq
% Multimedia Labratory, The Edward S. Rogers Sr. Dept of Electrical and
% Computer Engineering, University of Toronto

%% 
%************************************************************************
%       Fout    -> L: Length of signal
%               -> itt: number of iterations
%               -> Mod: Modulation Type (refer to 'Mapping.m' for allowed
%                        types)
%               -> M: M-ary, binary (2), quade (4) or 2^n
%               -> k: Number of bits per symbol
%               -> Block: String for block type (refer to 'Mapping.m')
%               -> BlockL: Length of blocks (refer to 'Mapping.m')
%               -> Numtx: Number of transmit antennas
%               -> Numrx: Number of receive antennas
%               -> SNR: a vector of SNR values
%               -> sBER: a temporary vector to store the computed BER
%               -> BER: a zero vector to store the computed BER
%               -> Out: a zero vector to store the final received signal
%                       to be be saved for reference
%               -> InSig: The input signal in bits
%               -> S: Numtx column vectors representing the split input
%                      signal
%               -> Saving: 'backup' for saving the data. 
%               Filter Parameters:
%               -> r: Roll-off-factor Used in raised cosine filter
%               -> T: Bit period
%               -> dur:  Duration
%               -> FltrType: The type of filter generated. 
%               -> nsamp: oversampling rate
%               Parameters added in this module:
%               -> InSym: Numtx column vectors of the modulated signal.
%               -> Px: Power of signal (1 for all psk, diff for QAM)
%**************************************************************************

%Which 'Types' are fully implemented as of the date above:
%BPSK, QPSK, 16PSK, 8PSK, 16QAM, 64QAM, DQPSK, DBPSK
%
% The table below explains the partially implemented
% ***Type***              ***Block***            
% 8DPSK                    "0 Symbol" or "Every L Symbols"
% 16DPSK                   "0 Symbol" or "Every L Symbols"
% pi2DBPSK                 "0 Symbol" or "Every L Symbols"
% pi4DQPSK                 "0 Symbol" or "Every L Symbols"
% Delayed_DBPSK            "0 Symbol" or "Every L Symbols"
% Delayed_DQPSK            "0 Symbol" or "Every L Symbols"
% AsymBPSK                        --            --
% AsymQPSK                        --            --
% AsymDBPSK                "0 Symbol" or "Every L Symbols"
% AsymDQPSK                "0 Symbol" or "Every L Symbols"
% AsymBPSK_BPSK                   --     "Every L Symbols"
% AsymQPSK_QPSK                   --     "Every L Symbols"
% AsymDBPSK_DBPSK                 --     "Every L Symbols"
% AsymDQPSK_DQPSK                 --     "Every L Symbols"
% 
% *******NOTE********
% For constellation other than Binary your input signal MUST be sized
% accordingly to generate corresponding symbole size. 
% ********
% For Quardature, length of "InputSig" must be Twice the desired length
% 8  -> 3*length(InputSig)
% 16 -> 4*length(InputSig)
% 64 -> 6*length(InputSig)


% 'BPSK','QPSK'(,'BPSK_End','QPSK_End')
% 'DBPSK','DQPSK'(,'DBPSK_End','DQPSK_End')
% 'pi2DBPSK','pi4DQPSK','Delayed_DBPSK','Delayed_DQPSK'
% 'AsymBPSK','AsymQPSK','AsymDBPSK','AsymDQPSK'
% 'AsymBPSK_BPSK','AsymQPSK_QPSK','AsymDBPSK_DBPSK','AsymDQPSK_DQPSK'
% 'BPSK_AsymBPSK_End','QPSK_AsymQPSK_End','DBPSK_AsymDBPSK_End','DQPSK_AsymDQPSK_End'
% '64MRDPSK','64MRDPSK_DQPSK_AsymDQPSK_End'
% 'DQPSK_64MRDPSK_AsymDQPSK','DQPSK_64MRDPSK_AsymDQPSK_End','DQPSK[2]_64MRDPSK_AsymDQPSK_End'
% 'DBPSK_64MRDPSK_AsymDBPSK','DBPSK_64MRDPSK_AsymDBPSK_End','DBPSK[2]_64MRDPSK_AsymDBPSK_End'
% '8PSK','8DPSK','DQPSK_8DPSK_AsymDQPSK_End','DBPSK_8DPSK_AsymDBPSK_End'
% 'DQPSK[2]_8DPSK_AsymDQPSK_End','DBPSK[2]_8DPSK_AsymDBPSK_End'
% '16PSK','16DPSK','DQPSK_16DPSK_AsymDQPSK_End','DBPSK_16DPSK_AsymDBPSK_End'
% 'DQPSK[2]_16DPSK_AsymDQPSK_End','DBPSK[2]_16DPSK_AsymDBPSK_End'
% '16QAM','DQPSK_16QAM_AsymDQPSK_End','DBPSK_16QAM_AsymDBPSK_End',
% 'DQPSK[2]_16QAM_AsymDQPSK_End','DBPSK[2]_16QAM_AsymDBPSK_End'
% '64QAM','DQPSK_64QAM_AsymDQPSK_End','DBPSK_64QAM_AsymDQPSK_End',
% 'DQPSK[2]_64QAM_AsymDQPSK_End','DBPSK[2]_64QAM_ASymDBPSK_End'


%% ========================= Code ================================
%=================================================================
addpath ./Modulation
Fout=Fin; 
temp=Fin;%this is used for the recursive cases
Fout.Px=1;% Power of the signal, 1 For BPSK. Different for other techniques

%Setting the Parameters
Length= length(Fout.S);
Type= Fout.Mod;
Block= Fout.Block;
L= Fout.BlockL;
Delay= 2; %only used in the delayed modulation techniques, must change here

te = isempty(findstr(Type,'End'));
be = isempty(findstr(Block,'End'));
if (te ~= be)
   error('*** GENERATE_DATA.M : Data Type does not match Block Type! ***');
end;

for ii= 1:Fout.Numtx
    Data.Bit=Fout.S(:,ii);

    switch (Type)
    case {'BPSK','BPSK_End'} %%% 'BPSK_End' SHOULD ONLY BE USED INTERNALLY
       % Modulate the inputsignal
       Data.Symbol = bpsk(Data.Bit);
    case {'QPSK','QPSK_End'} %%% 'QPSK_End' SHOULD ONLY BE USED INTERNALLY
       Data.Symbol = qpsk(Data.Bit);
    case {'DBPSK'}
       %%% Train[DBPSK] Data[DBPSK]
       switch (Block)
       case {'0 Symbol'}
          Data.Symbol = dbpsk(bpsk(Data.Bit));
       otherwise
          switch (Block)
          case {'Every L Symbols'}    
              [TrainBlock, DataBlock] = data_block(Block, L, Length);
          case {'Every L Symbols Beginning'}%not implemented in data_block    
              [TrainBlock, DataBlock] = data_block(Block, L, Length);          
          otherwise%Must change the required length in 'data_block' as required
              [TrainBlock, DataBlock] = data_block(Block);
          end    
          Data.Symbol = [];
          % size(TrainBlock, 1) shows how many groups of training symbols are there in a frame.
          for blk_cnt = 1 : size(TrainBlock,1)
             % Generates the data for the subportions in between the training symbol blocks.  The +1
             % is used to include the training symbol block.  The reason I need to split it up between
             % training symbols is that the training symbol itself needs not to be differentially encoded
             % because we know that symbol.  The KF just knows the symbol and don't need to DE decode it.
             % That's why that symbol is not included and we need to split this up.
             len = DataBlock(blk_cnt,2)-TrainBlock(blk_cnt,1)+1;
             blk_bit = Data.Bit(((blk_cnt-1)*len)+1: blk_cnt * len);
             blk_sym = dbpsk(bpsk(blk_bit));
             % Appends the subportions of the pilot and data blocks into a continuous 162 symbol block.
             % blk_bit and blk_sym contains both the user data and the training
             % data split into size(Trainblock, 1) subportions.
             Data.Symbol = [Data.Symbol, blk_sym];
          end;
       end;
    case {'DQPSK'}
       %%% Train[DQPSK] Data[DQPSK]
       switch (Block)
       case {'0 Symbol'}
          Data.Symbol = dqpsk(qpsk(Data.Bit));
       otherwise
          switch (Block)
          case {'Every L Symbols'}    
              [TrainBlock, DataBlock] = data_block(Block, L, Length);
          case {'Every L Symbols Beginning'}%not implemented in data_block    
              [TrainBlock, DataBlock] = data_block(Block, L, Length);          
          otherwise%Must change the required length in 'data_block' as required
              [TrainBlock, DataBlock] = data_block(Block);
          end    
          Data.Symbol = [];
          for blk_cnt = 1 : size(TrainBlock,1)
             len = DataBlock(blk_cnt,2)-TrainBlock(blk_cnt,1)+1;
             blk_bit = Data.Bit(((blk_cnt-1)*len)+1: blk_cnt *len);
             blk_sym = dqpsk(qpsk(blk_bit));
             Data.Symbol = [Data.Symbol, blk_sym];
          end;
       end;
    case {'DBPSK_End'}
       %%% Data[DBSPK] Train[DBPSK]
       %%% SHOULD ONLY BE USED INTERNALLY
       switch (Block)
       case {'0 Symbol'}
          Data.Bit = binary(Length);
          Data.Symbol = dbpsk(bpsk(Data.Bit));
       otherwise
          [TrainBlock, DataBlock] = data_block(Block);
          Data.Bit = [];
          Data.Symbol = [];
          for blk_cnt = 1 : size(TrainBlock,1)
             len = TrainBlock(blk_cnt,2)-DataBlock(blk_cnt,1)+1;
             blk_bit = binary(len);
             blk_sym = dbpsk(bpsk(blk_bit));
             Data.Bit = [Data.Bit, blk_bit];
             Data.Symbol = [Data.Symbol, blk_sym];
          end;
       end;


    case {'DQPSK_End'}
       %%% Data[DQPSK] Train[DQPSK]
       %%% SHOULD ONLY BE USED INTERNALLY
       switch (Block)
       case {'0 Symbol'}
          Data.Bit = binary(2*Length);
          Data.Symbol = dqpsk(qpsk(Data.Bit));
       otherwise
    %      [TrainBlock, DataBlock] = data_block(Block);
          switch (Block)
          case {'Every L Symbols', 'Every L Symbols End'}
              [TrainBlock, DataBlock] = data_block(Block, L, Length);
          case {'Every L Symbols Beginning', 'Every L Symbols Beginning End'}    
              [TrainBlock, DataBlock] = data_block(Block, L, Length);          
          otherwise
              [TrainBlock, DataBlock] = data_block(Block);
          end    
          Data.Bit = [];
          Data.Symbol = [];
          for blk_cnt = 1 : size(TrainBlock,1)
             len = TrainBlock(blk_cnt,2)-DataBlock(blk_cnt,1)+1;

    %         len

             blk_bit = binary(2*len);
             blk_sym = dqpsk(qpsk(blk_bit));
             Data.Bit = [Data.Bit, blk_bit];
             Data.Symbol = [Data.Symbol, blk_sym];
          end;
       end;


    case {'pi2DBPSK'}
       %%% Train[pi2DBPSK] Data[pi2DBPSK]
       switch (Block)
       case {'0 Symbol'}
          Data.Symbol = pi2dbpsk(bpsk(Data.Bit));
       otherwise
          [TrainBlock, DataBlock] = data_block(Block, L, Length);
          Data.Symbol = [];
          for blk_cnt = 1 : size(TrainBlock,1)
             len = DataBlock(blk_cnt,2)-TrainBlock(blk_cnt,1)+1;
             blk_bit = Data.Bit(((blk_cnt-1)*len)+1: blk_cnt * len);
             blk_sym = pi2dbpsk(bpsk(blk_bit));
             Data.Symbol = [Data.Symbol, blk_sym];
          end;
       end;
    case {'pi4DQPSK'}
       %%% Train[pi4DQPSK] Data[pi4DQPSK]
       switch (Block)
       case {'0 Symbol'}
          Data.Symbol = pi4dqpsk(qpsk(Data.Bit));
       otherwise
          [TrainBlock, DataBlock] = data_block(Block, L, Length);
          Data.Symbol = [];
          for blk_cnt = 1 : size(TrainBlock,1)
             len = DataBlock(blk_cnt,2)-TrainBlock(blk_cnt,1)+1;
             blk_bit = Data.Bit(((blk_cnt-1)*len)+1: blk_cnt * len);
             blk_sym = pi4dqpsk(qpsk(blk_bit));
             Data.Symbol = [Data.Symbol, blk_sym];
          end;
       end;
    case {'Delayed_DBPSK'}
       %%% Train[Delayed_DBPSK] Data[Delayed_DBPSK]
       switch (Block)
       case {'0 Symbol'}
          Data.Symbol = delayed_dbpsk(bpsk(Data.Bit),Delay);
       otherwise
          [TrainBlock, DataBlock] = data_block(Block, L, Length);
          Data.Symbol = [];
          for blk_cnt = 1 : size(TrainBlock,1)
             len = DataBlock(blk_cnt,2)-TrainBlock(blk_cnt,1)+1;
             blk_bit = Data.Bit(((blk_cnt-1)*len)+1: blk_cnt * len);
             blk_sym = delayed_dbpsk(bpsk(blk_bit),Delay);
             Data.Symbol = [Data.Symbol, blk_sym];
          end;
       end;
    case {'Delayed_DQPSK'}
       %%% Train[Delayed_DQPSK] Data[Delayed_DQPSK]
       switch (Block)
       case {'0 Symbol'}
          Data.Symbol = delayed_dqpsk(qpsk(Data.Bit),Delay);
       otherwise
          [TrainBlock, DataBlock] = data_block(Block, L, Length);
          Data.Symbol = [];
          for blk_cnt = 1 : size(TrainBlock,1)
             len = DataBlock(blk_cnt,2)-TrainBlock(blk_cnt,1)+1;
             blk_bit = Data.Bit(((blk_cnt-1)*len)+1: blk_cnt * len);
             blk_sym = delayed_dqpsk(qpsk(blk_bit),Delay);
             Data.Symbol = [Data.Symbol, blk_sym];
          end;
       end;
    case {'AsymBPSK'}
       %Introduce temp object for recursion
       temp.S=Data.Bit;
       temp.Numtx=1;
       temp.Mod='BPSK';
       Data = Mapping(temp);
       for pos = 1 : length(Data.InSym)
          % Looks at the value of each of the data symbol and then assign it the hard-coded value
          % of the ASC in Stephen's code.  All the symbols are ASC encoded, not just the last 1.
          Data.Symbol(pos) = bpsk_2_wbpsk(Data.InSym(pos));
       end;
    case {'AsymQPSK'}
       %Introduce temp object for recursion
       temp.S=Data.Bit;
       temp.Numtx=1;
       temp.Mod='QPSK';
       Data = Mapping(temp);
       for pos = 1 : length(Data.InSym)
          Data.Symbol(pos) = qpsk_2_wqpsk(Data.InSym(pos));
       end;
    case {'AsymDBPSK'}
       %%% Train[AsymDBPSK] Data[AsymDBPSK]
       %Introduce temp object for recursion
       temp.S=Data.Bit;
       temp.Numtx=1;
       temp.Mod='DBPSK';
       Data = Mapping(temp);
       for pos = 1 : length(Data.InSym)
          Data.Symbol(pos) = bpsk_2_wbpsk(Data.InSym(pos));
       end;
    case {'AsymDQPSK'}
       %%% Train[AsymDQPSK] Data[AsymDQPSK]
       %Introduce temp object for recursion
       temp.S=Data.Bit;
       temp.Numtx=1;
       temp.Mod='DQPSK';
       Data = Mapping(temp);
       for pos = 1 : length(Data.InSym)
          Data.Symbol(pos) = qpsk_2_wqpsk(Data.InSym(pos));
       end;
    case {'AsymBPSK_BPSK'}
       %%% Train[AsymBPSK] Data[BPSK]
       switch (Block)
       case {'0 Symbol'}
          error('*** GENERATE_DATA.M : Invalid Data Block Type! ***');
       otherwise
          %Introduce temp object for recursion
          temp.S=Data.Bit;
          temp.Numtx=1;
          temp.Mod='BPSK';
          Data = Mapping(temp); 
          Data.Symbol=Data.InSym;
          [TrainBlock, DataBlock] = data_block(Block, L, Length);
          for blk_cnt = 1 : size(TrainBlock,1)
             for pos = [TrainBlock(blk_cnt,1):TrainBlock(blk_cnt,2)]
                Data.Symbol(pos) = bpsk_2_wbpsk(Data.InSym(pos));
             end;
          end;
       end;
    case {'AsymQPSK_QPSK'}
       %%% Train[AsymQPSK] Data[QPSK]
       switch (Block)
       case {'0 Symbol'}
          error('*** GENERATE_DATA.M : Invalid Data Block Type! ***');
       otherwise
          %Introduce temp object for recursion
          temp.S=Data.Bit;
          temp.Numtx=1;
          temp.Mod='QPSK';
          Data = Mapping(temp); 
          Data.Symbol=Data.InSym;
          [TrainBlock, DataBlock] = data_block(Block, L, length(Data.InSym));
          for blk_cnt = 1 : size(TrainBlock,1)
             for pos = [TrainBlock(blk_cnt,1):TrainBlock(blk_cnt,2)]
                Data.Symbol(pos) = qpsk_2_wqpsk(Data.InSym(pos));
             end;
          end;
       end;
    case {'AsymDBPSK_DBPSK'}
       %%% Train[AsymDBPSK] Data[DBPSK]
       switch (Block)
       case {'0 Symbol'}
          error('*** GENERATE_DATA.M : Invalid Data Block Type! ***');
       otherwise
          %Introduce temp object for recursion
          temp.S=Data.Bit;
          temp.Numtx=1;
          temp.Mod='DBPSK';
          Data = Mapping(temp); 
          Data.Symbol=Data.InSym; 
          [TrainBlock, DataBlock] = data_block(Block, L, length(Data.InSym));
          for blk_cnt = 1 : size(TrainBlock,1)
             for pos = [TrainBlock(blk_cnt,1):TrainBlock(blk_cnt,2)]
                Data.Symbol(pos) = bpsk_2_wbpsk(Data.InSym(pos));
             end;
          end;
       end;
    case {'AsymDQPSK_DQPSK'}
       %%% Train[AsymDQPSK] Data[DQPSK]
       switch (Block)
       case {'0 Symbol'}
          error('*** GENERATE_DATA.M : Invalid Data Block Type! ***');
       otherwise
          %Introduce temp object for recursion
          temp.S=Data.Bit;
          temp.Numtx=1;
          temp.Mod='DQPSK';
          switch (Block)
          case {'Every L Symbols', 'Every L Symbols End'}
              Data = Mapping(temp); 
              Data.Symbol=Data.InSym; 
              [TrainBlock, DataBlock] = data_block(Block, L, length(Data.InSym));
          % case {'Every L Symbols Beginning'}
          % Data = generate_data('DQPSK', Block, Length, Delay, L, Amount);
          % [TrainBlock, DataBlock] = data_block(Block, L, Amount);          
          otherwise
              Data = Mapping(temp); 
              Data.Symbol=Data.InSym; 
              [TrainBlock, DataBlock] = data_block(Block);
          end    
          for blk_cnt = 1 : size(TrainBlock,1)
             for pos = [TrainBlock(blk_cnt,1):TrainBlock(blk_cnt,2)]
                Data.Symbol(pos) = qpsk_2_wqpsk(Data.InSym(pos));
             end;
          end;
       end;
    case {'BPSK_AsymBPSK_End'}
       %%% Data[BPSK] Train[AsymBPSK]
       switch (Block)
       case {'0 Symbol'}
          error('*** GENERATE_DATA.M : Invalid Data Block Type! ***');
       otherwise
          Data = generate_data('BPSK_End',Block,Length);
          [TrainBlock, DataBlock] = data_block(Block);
          for blk_cnt = 1 : size(TrainBlock,1)
             for pos = [TrainBlock(blk_cnt,1):TrainBlock(blk_cnt,2)]
                Data.Symbol(pos) = bpsk_2_wbpsk(Data.Symbol(pos));
             end;
          end;
       end;
    case {'QPSK_AsymQPSK_End'}
       %%% Data[QPSK] Train[AsymQPSK]
       switch (Block)
       case {'0 Symbol'}
          error('*** GENERATE_DATA.M : Invalid Data Block Type! ***');
       otherwise
          Data = generate_data('QPSK_End',Block,Length);
          [TrainBlock, DataBlock] = data_block(Block);
          for blk_cnt = 1 : size(TrainBlock,1)
             for pos = [TrainBlock(blk_cnt,1):TrainBlock(blk_cnt,2)]
                Data.Symbol(pos) = qpsk_2_wqpsk(Data.Symbol(pos));
             end;
          end;
       end;
    case {'DBPSK_AsymDBPSK_End'}
       %%% Data[DBPSK] Train[AsymDBPSK]
       switch (Block)
       case {'0 Symbol'}
          error('*** GENERATE_DATA.M : Invalid Data Block Type! ***');
       otherwise
          Data = generate_data('DBPSK_End',Block,Length);
          [TrainBlock, DataBlock] = data_block(Block);
          for blk_cnt = 1 : size(TrainBlock,1)
             for pos = [TrainBlock(blk_cnt,1):TrainBlock(blk_cnt,2)]
                Data.Symbol(pos) = bpsk_2_wbpsk(Data.Symbol(pos));
             end;
          end;
       end;


    case {'DQPSK_AsymDQPSK_End'}
       %%% Data[DQPSK] Train[AsymDQPSK]
       switch (Block)
       case {'0 Symbol'}
          error('*** GENERATE_DATA.M : Invalid Data Block Type! ***');
       otherwise
          %      [TrainBlock, DataBlock] = data_block(Block);
          switch (Block)
          case {'Every L Symbols', 'Every L Symbols End'}
              % Generates each subblock as DQPSK symbols first
              Data = generate_data('DQPSK_End', Block, Length, Delay, L);
              % We need this line again because the previous line only returns
              % the data symbols and not the positions of the trainblock and datablock.
              [TrainBlock, DataBlock] = data_block(Block, L);
          case {'Every L Symbols Beginning', 'Every L Symbols Beginning End'}
              Data = generate_data('DQPSK_End', Block, Length, Delay, L, Amount);
              [TrainBlock, DataBlock] = data_block(Block, L, Amount);          
          otherwise
              Data = generate_data('DQPSK_End', Block, Length, Delay);
              [TrainBlock, DataBlock] = data_block(Block);
          end    

          % Changes symbols marked trainblock in each pilot subblock to an ASC symbol

          for blk_cnt = 1 : size(TrainBlock,1)
             for pos = [TrainBlock(blk_cnt,1):TrainBlock(blk_cnt,2)]

    %            size(Data.Symbol)
    %            pos

                % Converts the symbols labelled in trainblock to the ASC constellation.
                Data.Symbol(pos) = qpsk_2_wqpsk(Data.Symbol(pos));
             end;
          end;
       end;



    case {'64MRDPSK'}
       %%% Train[64MRDPSK] Data[64MRDPSK]
       switch (Block)
       case {'0 Symbol'}
          Data.Bit = binary(6*Length);
          Data.Symbol = a64_mr_dpsk(Data.Bit);
       otherwise
          [TrainBlock, DataBlock] = data_block(Block);
          Data.Bit = [];
          Data.Symbol = [];
          for blk_cnt = 1 : size(TrainBlock,1)
             len = DataBlock(blk_cnt,2)-TrainBlock(blk_cnt,1)+1;
             blk_bit = binary(6*len);
             blk_sym = a64mrdpsk(blk_bit);
             Data.Bit = [Data.Bit, blk_bit];
             Data.Symbol = [Data.Symbol, blk_sym];
          end;
       end;
    case {'64MRDPSK_DQPSK_AsymDQPSK_End'}
       %%% Data[64MRDPSK,DQPSK] Train[AsymDQPSK]
       switch (Block)
       case {'0 Symbol'}
          error('*** GENERATE_DATA.M : Invalid Data Block Type! ***');
       otherwise
          [TrainBlock, DataBlock] = data_block(Block);
          Data.Bit = [];
          Data.Symbol = [];
          for blk_cnt = 1 : size(TrainBlock,1)
             DN = DataBlock(blk_cnt,2)-DataBlock(blk_cnt,1);
             QN = 1;
             WN = TrainBlock(blk_cnt,2)-TrainBlock(blk_cnt,1)+1;
             blk_bit = binary(6*DN+2*QN+2*WN);
             DBR = [1:6*DN];
             QBR = [6*DN+1:6*DN+2*QN];
             WBR = [6*DN+2*QN+1:6*DN+2*QN+2*WN];
             blk_sym = [a64mrdpsk(blk_bit(DBR)), dqpsk(qpsk(blk_bit([QBR, WBR])))];
             Data.Bit = [Data.Bit, blk_bit];
             Data.Symbol = [Data.Symbol, blk_sym];
             for pos = [TrainBlock(blk_cnt,1) : TrainBlock(blk_cnt,2)]
                Data.Symbol(pos) = qpsk_2_wqpsk(Data.Symbol(pos));
             end;
          end;
       end;
    case {'DQPSK_64MRDPSK_AsymDQPSK'}
       %%% Train[DQPSK] Data[64MRDPSK,AsymDQPSK]
       switch (Block)
       case {'0 Symbol'}
          error('*** GENERATE_DATA.M : Invalid Data Block Type! ***');
       otherwise
          [TrainBlock, DataBlock] = data_block(Block);
          Data.Bit = [];
          Data.Symbol = [];
          for blk_cnt = 1 : size(TrainBlock,1)
             QN = TrainBlock(blk_cnt,2)-TrainBlock(blk_cnt,1)+1;
             DN = DataBlock(blk_cnt,2)-DataBlock(blk_cnt,1);
             WN = 1;
             blk_bit = binary(2*QN+6*DN+2*WN);
             QBR = [1:2*QN];
             DBR = [2*QN+1:2*QN+6*DN];
             WBR = [2*QN+6*DN+1:2*QN+6*DN+2*WN];
             blk_sym = [dqpsk(qpsk(blk_bit(QBR))), a64mrdpsk(blk_bit(DBR)), dqpsk(qpsk(blk_bit(WBR)))];
             Data.Bit = [Data.Bit, blk_bit];
             Data.Symbol = [Data.Symbol, blk_sym];
             for pos = [DataBlock(blk_cnt,2)-WN+1:DataBlock(blk_cnt,2)]
                Data.Symbol(pos) = qpsk_2_wqpsk(Data.Symbol(pos));
             end;
          end;
       end;
    case {'DQPSK_64MRDPSK_AsymDQPSK_End'}
       %%% Data[DQPSK,64MRDPSK] Train[AsymDQPSK]
       switch (Block)
       case {'0 Symbol'}
          error('*** GENERATE_DATA.M : Invalid Data Block Type! ***');
       otherwise
          [TrainBlock, DataBlock] = data_block(Block);
          Data.Bit = [];
          Data.Symbol = [];
          for blk_cnt = 1 : size(TrainBlock,1)
             QN = 1;
             DN = DataBlock(blk_cnt,2)-DataBlock(blk_cnt,1);
             WN = TrainBlock(blk_cnt,2)-TrainBlock(blk_cnt,1)+1;
             blk_bit = binary(2*QN+6*DN+2*WN);
             QBR = [1:2*QN];
             DBR = [2*QN+1:2*QN+6*DN];
             WBR = [2*QN+6*DN+1:2*QN+6*DN+2*WN];
             blk_sym = [dqpsk(qpsk(blk_bit(QBR))), a64mrdpsk(blk_bit(DBR)), dqpsk(qpsk(blk_bit(WBR)))];
             Data.Bit = [Data.Bit, blk_bit];
             Data.Symbol = [Data.Symbol, blk_sym];
             for pos = [TrainBlock(blk_cnt,1):TrainBlock(blk_cnt,2)]
                Data.Symbol(pos) = qpsk_2_wqpsk(Data.Symbol(pos));
             end;
          end;
       end;
    case {'DQPSK[2]_64MRDPSK_AsymDQPSK_End'}
       %%% Data[DQPSK[2],64MRDPSK] Train[AsymDQPSK]
       switch (Block)
       case {'0 Symbol'}
          error('*** GENERATE_DATA.M : Invalid Data Block Type! ***');
       otherwise
          [TrainBlock, DataBlock] = data_block(Block);
          Data.Bit = [];
          Data.Symbol = [];
          for blk_cnt = 1 : size(TrainBlock,1)
             QN = 2;
             DN = DataBlock(blk_cnt,2)-DataBlock(blk_cnt,1)-1;
             WN = TrainBlock(blk_cnt,2)-TrainBlock(blk_cnt,1)+1;
             blk_bit = binary(2*QN+6*DN+2*WN);
             QBR = [1:2*QN];
             DBR = [2*QN+1:2*QN+6*DN];
             WBR = [2*QN+6*DN+1:2*QN+6*DN+2*WN];
             blk_sym = [dqpsk(qpsk(blk_bit(QBR))), a64mrdpsk(blk_bit(DBR)), dqpsk(qpsk(blk_bit(WBR)))];
             Data.Bit = [Data.Bit, blk_bit];
             Data.Symbol = [Data.Symbol, blk_sym];
             for pos = [TrainBlock(blk_cnt,1):TrainBlock(blk_cnt,2)]
                Data.Symbol(pos) = qpsk_2_wqpsk(Data.Symbol(pos));
             end;
          end;
       end;
    case {'DBPSK_64MRDPSK_AsymDBPSK'}
       %%% Train[DBPSK] Data[64MRDPSK,AsymDBPSK]
       switch (Block)
       case {'0 Symbol'}
          error('*** GENERATE_DATA.M : Invalid Data Block Type! ***');
       otherwise
          [TrainBlock, DataBlock] = data_block(Block);
          Data.Bit = [];
          Data.Symbol = [];
          for blk_cnt = 1 : size(TrainBlock,1)
             QN = TrainBlock(blk_cnt,2)-TrainBlock(blk_cnt,1)+1;
             DN = DataBlock(blk_cnt,2)-DataBlock(blk_cnt,1);
             WN = 1;
             blk_bit = binary(1*QN+6*DN+1*WN);
             QBR = [1:1*QN];
             DBR = [1*QN+1:1*QN+6*DN];
             WBR = [1*QN+6*DN+1:1*QN+6*DN+1*WN];
             blk_sym = [dbpsk(bpsk(blk_bit(QBR))), a64mrdpsk(blk_bit(DBR)), dbpsk(bpsk(blk_bit(WBR)))];
             Data.Bit = [Data.Bit, blk_bit];
             Data.Symbol = [Data.Symbol, blk_sym];
             for pos = [DataBlock(blk_cnt,2)-WN+1:DataBlock(blk_cnt,2)]
                Data.Symbol(pos) = bpsk_2_wbpsk(Data.Symbol(pos));
             end;
          end;
       end;
    case {'DBPSK_64MRDPSK_AsymDBPSK_End'}
       %%% Data[DBPSK,64MRDPSK] Train[AsymDBPSK]
       switch (Block)
       case {'0 Symbol'}
          error('*** GENERATE_DATA.M : Invalid Data Block Type! ***');
       otherwise
          [TrainBlock, DataBlock] = data_block(Block);
          Data.Bit = [];
          Data.Symbol = [];
          for blk_cnt = 1 : size(TrainBlock,1)
             QN = 1;
             DN = DataBlock(blk_cnt,2)-DataBlock(blk_cnt,1);
             WN = TrainBlock(blk_cnt,2)-TrainBlock(blk_cnt,1)+1;
             blk_bit = binary(1*QN+6*DN+1*WN);
             QBR = [1:1*QN];
             DBR = [1*QN+1:1*QN+6*DN];
             WBR = [1*QN+6*DN+1:1*QN+6*DN+1*WN];
             blk_sym = [dbpsk(bpsk(blk_bit(QBR))), a64mrdpsk(blk_bit(DBR)), dbpsk(bpsk(blk_bit(WBR)))];
             Data.Bit = [Data.Bit, blk_bit];
             Data.Symbol = [Data.Symbol, blk_sym];
             for pos = [TrainBlock(blk_cnt,1):TrainBlock(blk_cnt,2)]
                Data.Symbol(pos) = bpsk_2_wbpsk(Data.Symbol(pos));
             end;
          end;
       end;
    case {'DBPSK[2]_64MRDPSK_AsymDBPSK_End'}
       %%% Data[DBPSK[2],64MRDPSK] Train[AsymDBPSK]
       switch (Block)
       case {'0 Symbol'}
          error('*** GENERATE_DATA.M : Invalid Data Block Type! ***');
       otherwise
          [TrainBlock, DataBlock] = data_block(Block);
          Data.Bit = [];
          Data.Symbol = [];
          for blk_cnt = 1 : size(TrainBlock,1)
             QN = 2;
             DN = DataBlock(blk_cnt,2)-DataBlock(blk_cnt,1)-1;
             WN = TrainBlock(blk_cnt,2)-TrainBlock(blk_cnt,1)+1;
             blk_bit = binary(1*QN+6*DN+1*WN);
             QBR = [1:1*QN];
             DBR = [1*QN+1:1*QN+6*DN];
             WBR = [1*QN+6*DN+1:1*QN+6*DN+1*WN];
             blk_sym = [dbpsk(bpsk(blk_bit(QBR))), a64mrdpsk(blk_bit(DBR)), dbpsk(bpsk(blk_bit(WBR)))];
             Data.Bit = [Data.Bit, blk_bit];
             Data.Symbol = [Data.Symbol, blk_sym];
             for pos = [TrainBlock(blk_cnt,1):TrainBlock(blk_cnt,2)]
                Data.Symbol(pos) = bpsk_2_wbpsk(Data.Symbol(pos));
             end;
          end;
       end;
    case {'8PSK'} 
       Data.Symbol = a8psk(Data.Bit);
    case {'8DPSK'}
       %%% Train[8DPSK] Data[8DPSK]
       switch (Block)
       case {'0 Symbol'}
          Data.Symbol = a8dpsk(a8psk(Data.Bit));
       otherwise
          [TrainBlock, DataBlock] = data_block(Block);
          Data.Symbol = [];
          for blk_cnt = 1 : size(TrainBlock,1)
             len = DataBlock(blk_cnt,2)-TrainBlock(blk_cnt,1)+1;
             blk_bit = Data.Bit(((blk_cnt-1)*len)+1: blk_cnt * len);
             blk_sym = a8dpsk(a8psk(blk_bit));
             Data.Symbol = [Data.Symbol, blk_sym];
          end;
       end;
    case {'DQPSK_8DPSK_AsymDQPSK_End'}
       %%% Data[DQPSK,8DPSK] Train[AsymDQPSK]
       switch (Block)
       case {'0 Symbol'}
          error('*** GENERATE_DATA.M : Invalid Data Block Type! ***');
       otherwise
          [TrainBlock, DataBlock] = data_block(Block);
          Data.Bit = [];
          Data.Symbol = [];
          for blk_cnt = 1 : size(TrainBlock,1)
             QN = 1;
             DN = DataBlock(blk_cnt,2)-DataBlock(blk_cnt,1);
             WN = TrainBlock(blk_cnt,2)-TrainBlock(blk_cnt,1)+1;
             blk_bit = binary(2*QN+3*DN+2*WN);
             QBR = [1:2*QN];
             DBR = [2*QN+1:2*QN+3*DN];
             WBR = [2*QN+3*DN+1:2*QN+3*DN+2*WN];
             blk_sym = [dqpsk(qpsk(blk_bit(QBR))), a8dpsk(a8psk(blk_bit(DBR))), dqpsk(qpsk(blk_bit(WBR)))];
             Data.Bit = [Data.Bit, blk_bit];
             Data.Symbol = [Data.Symbol, blk_sym];
             for pos = [TrainBlock(blk_cnt,1):TrainBlock(blk_cnt,2)]
                Data.Symbol(pos) = qpsk_2_wqpsk(Data.Symbol(pos));
             end;
          end;
       end;
    case {'DBPSK_8DPSK_AsymDBPSK_End'}
       %%% Data[DBPSK,8DPSK] Train[AsymDBPSK]
       switch (Block)
       case {'0 Symbol'}
          error('*** GENERATE_DATA.M : Invalid Data Block Type! ***');
       otherwise
          [TrainBlock, DataBlock] = data_block(Block);
          Data.Bit = [];
          Data.Symbol = [];
          for blk_cnt = 1 : size(TrainBlock,1)
             QN = 1;
             DN = DataBlock(blk_cnt,2)-DataBlock(blk_cnt,1);
             WN = TrainBlock(blk_cnt,2)-TrainBlock(blk_cnt,1)+1;
             blk_bit = binary(1*QN+3*DN+1*WN);
             QBR = [1:1*QN];
             DBR = [1*QN+1:1*QN+3*DN];
             WBR = [1*QN+3*DN+1:1*QN+3*DN+1*WN];
             blk_sym = [dbpsk(bpsk(blk_bit(QBR))), a8dpsk(a8psk(blk_bit(DBR))), dbpsk(bpsk(blk_bit(WBR)))];
             Data.Bit = [Data.Bit, blk_bit];
             Data.Symbol = [Data.Symbol, blk_sym];
             for pos = [TrainBlock(blk_cnt,1):TrainBlock(blk_cnt,2)]
                Data.Symbol(pos) = bpsk_2_wbpsk(Data.Symbol(pos));
             end;
          end;
       end;
    case {'DQPSK[2]_8DPSK_AsymDQPSK_End'}
       %%% Data[DQPSK[2],8DPSK] Train[AsymDQPSK]
       switch (Block)
       case {'0 Symbol'}
          error('*** GENERATE_DATA.M : Invalid Data Block Type! ***');
       otherwise
          [TrainBlock, DataBlock] = data_block(Block);
          Data.Bit = [];
          Data.Symbol = [];
          for blk_cnt = 1 : size(TrainBlock,1)
             QN = 2;
             DN = DataBlock(blk_cnt,2)-DataBlock(blk_cnt,1)-1;
             WN = TrainBlock(blk_cnt,2)-TrainBlock(blk_cnt,1)+1;
             blk_bit = binary(2*QN+3*DN+2*WN);
             QBR = [1:2*QN];
             DBR = [2*QN+1:2*QN+3*DN];
             WBR = [2*QN+3*DN+1:2*QN+3*DN+2*WN];
             blk_sym = [dqpsk(qpsk(blk_bit(QBR))), a8dpsk(a8psk(blk_bit(DBR))), dqpsk(qpsk(blk_bit(WBR)))];
             Data.Bit = [Data.Bit, blk_bit];
             Data.Symbol = [Data.Symbol, blk_sym];
             for pos = [TrainBlock(blk_cnt,1):TrainBlock(blk_cnt,2)]
                Data.Symbol(pos) = qpsk_2_wqpsk(Data.Symbol(pos));
             end;
          end;
       end;
    case {'DBPSK[2]_8DPSK_AsymDBPSK_End'}
       %%% Data[DBPSK[2],8DPSK] Train[AsymDBPSK]
       switch (Block)
       case {'0 Symbol'}
          error('*** GENERATE_DATA.M : Invalid Data Block Type! ***');
       otherwise
          [TrainBlock, DataBlock] = data_block(Block);
          Data.Bit = [];
          Data.Symbol = [];
          for blk_cnt = 1 : size(TrainBlock,1)
             QN = 2;
             DN = DataBlock(blk_cnt,2)-DataBlock(blk_cnt,1)-1;
             WN = TrainBlock(blk_cnt,2)-TrainBlock(blk_cnt,1)+1;
             blk_bit = binary(1*QN+3*DN+1*WN);
             QBR = [1:1*QN];
             DBR = [1*QN+1:1*QN+3*DN];
             WBR = [1*QN+3*DN+1:1*QN+3*DN+1*WN];
             blk_sym = [dbpsk(bpsk(blk_bit(QBR))), a8dpsk(a8psk(blk_bit(DBR))), dbpsk(bpsk(blk_bit(WBR)))];
             Data.Bit = [Data.Bit, blk_bit];
             Data.Symbol = [Data.Symbol, blk_sym];
             for pos = [TrainBlock(blk_cnt,1):TrainBlock(blk_cnt,2)]
                Data.Symbol(pos) = bpsk_2_wbpsk(Data.Symbol(pos));
             end;
          end;
       end;
    case {'16PSK'}
       Data.Symbol = a16psk(Data.Bit);
    case {'16DPSK'}
       %%% Train[16DPSK] Data[16DPSK]
       switch (Block)
       case {'0 Symbol'}
          Data.Symbol = a16dpsk(a16psk(Data.Bit));
       otherwise
          [TrainBlock, DataBlock] = data_block(Block);
          Data.Symbol = [];
          for blk_cnt = 1 : size(TrainBlock,1)
             len = DataBlock(blk_cnt,2)-TrainBlock(blk_cnt,1)+1;
             blk_bit = Data.Bit(((blk_cnt-1)*len)+1: blk_cnt * len);
             blk_sym = a16dpsk(a16psk(blk_bit));
             Data.Symbol = [Data.Symbol, blk_sym];
          end;
       end;
    case {'DQPSK_16DPSK_AsymDQPSK_End'}
       %%% Data[DQPSK,16DPSK] Train[AsymDQPSK]
       switch (Block)
       case {'0 Symbol'}
          error('*** GENERATE_DATA.M : Invalid Data Block Type! ***');
       otherwise
          [TrainBlock, DataBlock] = data_block(Block);
          Data.Bit = [];
          Data.Symbol = [];
          for blk_cnt = 1 : size(TrainBlock,1)
             QN = 1;
             DN = DataBlock(blk_cnt,2)-DataBlock(blk_cnt,1);
             WN = TrainBlock(blk_cnt,2)-TrainBlock(blk_cnt,1)+1;
             blk_bit = binary(2*QN+4*DN+2*WN);
             QBR = [1:2*QN];
             DBR = [2*QN+1:2*QN+4*DN];
             WBR = [2*QN+4*DN+1:2*QN+4*DN+2*WN];
             blk_sym = [dqpsk(qpsk(blk_bit(QBR))), a16dpsk(a16psk(blk_bit(DBR))), dqpsk(qpsk(blk_bit(WBR)))];
             Data.Bit = [Data.Bit, blk_bit];
             Data.Symbol = [Data.Symbol, blk_sym];
             for pos = [TrainBlock(blk_cnt,1):TrainBlock(blk_cnt,2)]
                Data.Symbol(pos) = qpsk_2_wqpsk(Data.Symbol(pos));
             end;
          end;
       end;
    case {'DBPSK_16DPSK_AsymDBPSK_End'}
       %%% Data[DBPSK,16DPSK] Train[AsymDBPSK]
       switch (Block)
       case {'0 Symbol'}
          error('*** GENERATE_DATA.M : Invalid Data Block Type! ***');
       otherwise
          [TrainBlock, DataBlock] = data_block(Block);
          Data.Bit = [];
          Data.Symbol = [];
          for blk_cnt = 1 : size(TrainBlock,1)
             QN = 1;
             DN = DataBlock(blk_cnt,2)-DataBlock(blk_cnt,1);
             WN = TrainBlock(blk_cnt,2)-TrainBlock(blk_cnt,1)+1;
             blk_bit = binary(1*QN+4*DN+1*WN);
             QBR = [1:1*QN];
             DBR = [1*QN+1:1*QN+4*DN];
             WBR = [1*QN+4*DN+1:1*QN+4*DN+1*WN];
             blk_sym = [dbpsk(bpsk(blk_bit(QBR))), a16dpsk(a16psk(blk_bit(DBR))), dbpsk(bpsk(blk_bit(WBR)))];
             Data.Bit = [Data.Bit, blk_bit];
             Data.Symbol = [Data.Symbol, blk_sym];
             for pos = [TrainBlock(blk_cnt,1):TrainBlock(blk_cnt,2)]
                Data.Symbol(pos) = bpsk_2_wbpsk(Data.Symbol(pos));
             end;
          end;
       end;
    case {'DQPSK[2]_16DPSK_AsymDQPSK_End'}
       %%% Data[DQPSK[2],16DPSK] Train[AsymDQPSK]
       switch (Block)
       case {'0 Symbol'}
          error('*** GENERATE_DATA.M : Invalid Data Block Type! ***');
       otherwise
          [TrainBlock, DataBlock] = data_block(Block);
          Data.Bit = [];
          Data.Symbol = [];
          for blk_cnt = 1 : size(TrainBlock,1)
             QN = 2;
             DN = DataBlock(blk_cnt,2)-DataBlock(blk_cnt,1)-1;
             WN = TrainBlock(blk_cnt,2)-TrainBlock(blk_cnt,1)+1;
             blk_bit = binary(2*QN+4*DN+2*WN);
             QBR = [1:2*QN];
             DBR = [2*QN+1:2*QN+4*DN];
             WBR = [2*QN+4*DN+1:2*QN+4*DN+2*WN];
             blk_sym = [dqpsk(qpsk(blk_bit(QBR))), a16dpsk(a16psk(blk_bit(DBR))), dqpsk(qpsk(blk_bit(WBR)))];
             Data.Bit = [Data.Bit, blk_bit];
             Data.Symbol = [Data.Symbol, blk_sym];
             for pos = [TrainBlock(blk_cnt,1):TrainBlock(blk_cnt,2)]
                Data.Symbol(pos) = qpsk_2_wqpsk(Data.Symbol(pos));
             end;
          end;
       end;
    case {'DBPSK[2]_16DPSK_AsymDBPSK_End'}
       %%% Data[DBPSK[2],16DPSK] Train[AsymDBPSK]
       switch (Block)
       case {'0 Symbol'}
          error('*** GENERATE_DATA.M : Invalid Data Block Type! ***');
       otherwise
          [TrainBlock, DataBlock] = data_block(Block);
          Data.Bit = [];
          Data.Symbol = [];
          for blk_cnt = 1 : size(TrainBlock,1)
             QN = 2;
             DN = DataBlock(blk_cnt,2)-DataBlock(blk_cnt,1)-1;
             WN = TrainBlock(blk_cnt,2)-TrainBlock(blk_cnt,1)+1;
             blk_bit = binary(1*QN+4*DN+1*WN);
             QBR = [1:1*QN];
             DBR = [1*QN+1:1*QN+4*DN];
             WBR = [1*QN+4*DN+1:1*QN+4*DN+1*WN];
             blk_sym = [dbpsk(bpsk(blk_bit(QBR))), a16dpsk(a16psk(blk_bit(DBR))), dbpsk(bpsk(blk_bit(WBR)))];
             Data.Bit = [Data.Bit, blk_bit];
             Data.Symbol = [Data.Symbol, blk_sym];
             for pos = [TrainBlock(blk_cnt,1):TrainBlock(blk_cnt,2)]
                Data.Symbol(pos) = bpsk_2_wbpsk(Data.Symbol(pos));
             end;
          end;
       end;
    case {'16QAM'}
       %%% Train[16QAM] Data[16QAM]
       Data.Symbol = a16qam(Data.Bit);
       Fout.Px= 10;
    case {'DQPSK_16QAM_AsymDQPSK_End'}
       %%% Data[DQPSK,16QAM] Train[AsymDQPSK]
       switch (Block)
       case {'0 Symbol'}
          error('*** GENERATE_DATA.M : Invalid Data Block Type! ***');
       otherwise
          [TrainBlock, DataBlock] = data_block(Block);
          Data.Bit = [];
          Data.Symbol = [];
          for blk_cnt = 1 : size(TrainBlock,1)
             QN = 1;
             DN = DataBlock(blk_cnt,2)-DataBlock(blk_cnt,1);
             WN = TrainBlock(blk_cnt,2)-TrainBlock(blk_cnt,1)+1;
             blk_bit = binary(2*QN+4*DN+2*WN);
             QBR = [1:2*QN];
             DBR = [2*QN+1:2*QN+4*DN];
             WBR = [2*QN+4*DN+1:2*QN+4*DN+2*WN];
             blk_sym = [dqpsk(qpsk(blk_bit(QBR))), a16qam(blk_bit(DBR)), dqpsk(qpsk(blk_bit(WBR)))];
             Data.Bit = [Data.Bit, blk_bit];
             Data.Symbol = [Data.Symbol, blk_sym];
             for pos = [TrainBlock(blk_cnt,1):TrainBlock(blk_cnt,2)]
                Data.Symbol(pos) = qpsk_2_wqpsk(Data.Symbol(pos));
             end;
          end;
       end;
    case {'DBPSK_16QAM_AsymDBPSK_End'}
       %%% Data[DBPSK,16QAM] Train[AsymDBPSK]
       switch (Block)
       case {'0 Symbol'}
          error('*** GENERATE_DATA.M : Invalid Data Block Type! ***');
       otherwise
          [TrainBlock, DataBlock] = data_block(Block);
          Data.Bit = [];
          Data.Symbol = [];
          for blk_cnt = 1 : size(TrainBlock,1)
             QN = 1;
             DN = DataBlock(blk_cnt,2)-DataBlock(blk_cnt,1);
             WN = TrainBlock(blk_cnt,2)-TrainBlock(blk_cnt,1)+1;
             blk_bit = binary(1*QN+4*DN+1*WN);
             QBR = [1:1*QN];
             DBR = [1*QN+1:1*QN+4*DN];
             WBR = [1*QN+4*DN+1:1*QN+4*DN+1*WN];
             blk_sym = [dbpsk(bpsk(blk_bit(QBR))), a16qam(blk_bit(DBR)), dbpsk(bpsk(blk_bit(WBR)))];
             Data.Bit = [Data.Bit, blk_bit];
             Data.Symbol = [Data.Symbol, blk_sym];
             for pos = [TrainBlock(blk_cnt,1):TrainBlock(blk_cnt,2)]
                Data.Symbol(pos) = bpsk_2_wbpsk(Data.Symbol(pos));
             end;
          end;
       end;
    case {'DQPSK[2]_16QAM_AsymDQPSK_End'}
       %%% Data[DQPSK[2],16QAM] Train[AsymDQPSK]
       switch (Block)
       case {'0 Symbol'}
          error('*** GENERATE_DATA.M : Invalid Data Block Type! ***');
       otherwise
          [TrainBlock, DataBlock] = data_block(Block);
          Data.Bit = [];
          Data.Symbol = [];
          for blk_cnt = 1 : size(TrainBlock,1)
             QN = 2;
             DN = DataBlock(blk_cnt,2)-DataBlock(blk_cnt,1)-1;
             WN = TrainBlock(blk_cnt,2)-TrainBlock(blk_cnt,1)+1;
             blk_bit = binary(2*QN+4*DN+2*WN);
             QBR = [1:2*QN];
             DBR = [2*QN+1:2*QN+4*DN];
             WBR = [2*QN+4*DN+1:2*QN+4*DN+2*WN];
             blk_sym = [dqpsk(qpsk(blk_bit(QBR))), a16qam(blk_bit(DBR)), dqpsk(qpsk(blk_bit(WBR)))];
             Data.Bit = [Data.Bit, blk_bit];
             Data.Symbol = [Data.Symbol, blk_sym];
             for pos = [TrainBlock(blk_cnt,1):TrainBlock(blk_cnt,2)]
                Data.Symbol(pos) = qpsk_2_wqpsk(Data.Symbol(pos));
             end;
          end;
       end;
    case {'DBPSK[2]_16QAM_AsymDBPSK_End'}
       %%% Data[DBPSK[2],16QAM] Train[AsymDBPSK]
       switch (Block)
       case {'0 Symbol'}
          error('*** GENERATE_DATA.M : Invalid Data Block Type! ***');
       otherwise
          [TrainBlock, DataBlock] = data_block(Block);
          Data.Bit = [];
          Data.Symbol = [];
          for blk_cnt = 1 : size(TrainBlock,1)
             QN = 2;
             DN = DataBlock(blk_cnt,2)-DataBlock(blk_cnt,1)-1;
             WN = TrainBlock(blk_cnt,2)-TrainBlock(blk_cnt,1)+1;
             blk_bit = binary(1*QN+4*DN+1*WN);
             QBR = [1:1*QN];
             DBR = [1*QN+1:1*QN+4*DN];
             WBR = [1*QN+4*DN+1:1*QN+4*DN+1*WN];
             blk_sym = [dbpsk(bpsk(blk_bit(QBR))), a16qam(blk_bit(DBR)), dbpsk(bpsk(blk_bit(WBR)))];
             Data.Bit = [Data.Bit, blk_bit];
             Data.Symbol = [Data.Symbol, blk_sym];
             for pos = [TrainBlock(blk_cnt,1):TrainBlock(blk_cnt,2)]
                Data.Symbol(pos) = bpsk_2_wbpsk(Data.Symbol(pos));
             end;
          end;
       end;
    case {'64QAM'}
       %%% Train[64QAM] Data[64QAM]
       Data.Symbol = a64qam(Data.Bit);
       Fout.Px= 42;
    case {'DQPSK_64QAM_AsymDQPSK_End'}
       %%% Data[DQPSK,64QAM] Train[AsymDQPSK]
       switch (Block)
       case {'0 Symbol'}
          error('*** GENERATE_DATA.M : Invalid Data Block Type! ***');
       otherwise
          [TrainBlock, DataBlock] = data_block(Block);
          Data.Bit = [];
          Data.Symbol = [];
          for blk_cnt = 1 : size(TrainBlock,1)
             QN = 1;
             DN = DataBlock(blk_cnt,2)-DataBlock(blk_cnt,1);
             WN = TrainBlock(blk_cnt,2)-TrainBlock(blk_cnt,1)+1;
             blk_bit = binary(2*QN+6*DN+2*WN);
             QBR = [1:2*QN];
             DBR = [2*QN+1:2*QN+6*DN];
             WBR = [2*QN+6*DN+1:2*QN+6*DN+2*WN];
             blk_sym = [dqpsk(qpsk(blk_bit(QBR))), a64qam(blk_bit(DBR)), dqpsk(qpsk(blk_bit(WBR)))];
             Data.Bit = [Data.Bit, blk_bit];
             Data.Symbol = [Data.Symbol, blk_sym];
             for pos = [TrainBlock(blk_cnt,1):TrainBlock(blk_cnt,2)]
                Data.Symbol(pos) = qpsk_2_wqpsk(Data.Symbol(pos));
             end;
          end;
       end;
    case {'DBPSK_64QAM_AsymDBPSK_End'}
       %%% Data[DBPSK,64QAM] Train[AsymDBPSK]
       switch (Block)
       case {'0 Symbol'}
          error('*** GENERATE_DATA.M : Invalid Data Block Type! ***');
       otherwise
          [TrainBlock, DataBlock] = data_block(Block);
          Data.Bit = [];
          Data.Symbol = [];
          for blk_cnt = 1 : size(TrainBlock,1)
             QN = 1;
             DN = DataBlock(blk_cnt,2)-DataBlock(blk_cnt,1);
             WN = TrainBlock(blk_cnt,2)-TrainBlock(blk_cnt,1)+1;
             blk_bit = binary(1*QN+6*DN+1*WN);
             QBR = [1:1*QN];
             DBR = [1*QN+1:1*QN+6*DN];
             WBR = [1*QN+6*DN+1:1*QN+6*DN+1*WN];
             blk_sym = [dbpsk(bpsk(blk_bit(QBR))), a64qam(blk_bit(DBR)), dbpsk(bpsk(blk_bit(WBR)))];
             Data.Bit = [Data.Bit, blk_bit];
             Data.Symbol = [Data.Symbol, blk_sym];
             for pos = [TrainBlock(blk_cnt,1):TrainBlock(blk_cnt,2)]
                Data.Symbol(pos) = bpsk_2_wbpsk(Data.Symbol(pos));
             end;
          end;
       end;
    case {'DQPSK[2]_64QAM_AsymDQPSK_End'}
       %%% Data[DQPSK[2],64QAM] Train[AsymDQPSK]
       switch (Block)
       case {'0 Symbol'}
          error('*** GENERATE_DATA.M : Invalid Data Block Type! ***');
       otherwise
          [TrainBlock, DataBlock] = data_block(Block);
          Data.Bit = [];
          Data.Symbol = [];
          for blk_cnt = 1 : size(TrainBlock,1)
             QN = 2;
             DN = DataBlock(blk_cnt,2)-DataBlock(blk_cnt,1)-1;
             WN = TrainBlock(blk_cnt,2)-TrainBlock(blk_cnt,1)+1;
             blk_bit = binary(2*QN+6*DN+2*WN);
             QBR = [1:2*QN];
             DBR = [2*QN+1:2*QN+6*DN];
             WBR = [2*QN+6*DN+1:2*QN+6*DN+2*WN];
             blk_sym = [dqpsk(qpsk(blk_bit(QBR))), a64qam(blk_bit(DBR)), dqpsk(qpsk(blk_bit(WBR)))];
             Data.Bit = [Data.Bit, blk_bit];
             Data.Symbol = [Data.Symbol, blk_sym];
             for pos = [TrainBlock(blk_cnt,1):TrainBlock(blk_cnt,2)]
                Data.Symbol(pos) = qpsk_2_wqpsk(Data.Symbol(pos));
             end;
          end;
       end;
    case {'DBPSK[2]_64QAM_AsymDBPSK_End'}
       %%% Data[DBPSK[2],64QAM] Train[AsymDBPSK]
       switch (Block)
       case {'0 Symbol'}
          error('*** GENERATE_DATA.M : Invalid Data Block Type! ***');
       otherwise
          [TrainBlock, DataBlock] = data_block(Block);
          Data.Bit = [];
          Data.Symbol = [];
          for blk_cnt = 1 : size(TrainBlock,1)
             QN = 2;
             DN = DataBlock(blk_cnt,2)-DataBlock(blk_cnt,1)-1;
             WN = TrainBlock(blk_cnt,2)-TrainBlock(blk_cnt,1)+1;
             blk_bit = binary(1*QN+6*DN+1*WN);
             QBR = [1:1*QN];
             DBR = [1*QN+1:1*QN+6*DN];
             WBR = [1*QN+6*DN+1:1*QN+6*DN+1*WN];
             blk_sym = [dbpsk(bpsk(blk_bit(QBR))), a64qam(blk_bit(DBR)), dbpsk(bpsk(blk_bit(WBR)))];
             Data.Bit = [Data.Bit, blk_bit];
             Data.Symbol = [Data.Symbol, blk_sym];
             for pos = [TrainBlock(blk_cnt,1):TrainBlock(blk_cnt,2)]
                Data.Symbol(pos) = bpsk_2_wbpsk(Data.Symbol(pos));
             end;
          end;
       end;
    % 'WQPSK_Z120','WQPSK_16QAM','WQPSK_16QPSK','WQPSK_C','WQPSK_Boost','QPSK_Boost'
    % 'WDQPSK_Z120','WDQPSK_16QAM','WDQPSK_16QPSK','WDQPSK_C','WDQPSK_Boost','DQPSK_Boost'
    % 'WQPSK_Boost2'
    % 'WDQPSK_Boost2'
    % 'WQPSK_Alt'
    % 'WDQPSK_Alt'
    % 'WDQPSK_DQPSK_64MRDPSK'
    case {'WQPSK_Z120','WQPSK_16QAM','WQPSK_16QPSK','WQPSK_C','WQPSK_Boost','QPSK_Boost'}
       %%% WARNING!!!
       error('*** GENERATE_DATA.M : Not Implemented Yet! ***');
       Data.Bit = binary(2*Data.Length);
       Data.Symbol = qpsk(Data.Bit);
       switch (Data.Block)
       case {'0 Symbol'}
       otherwise
          [TrainBlock, DataBlock] = data_block(Data.Block);
       end;
       for blk_cnt = 1 : size(TrainBlock,1)
          for pos = [TrainBlock(blk_cnt,1):TrainBlock(blk_cnt,2)]
             switch (Data.Symbol(pos))
             case {+1}
                index = 1;
             case {+j}
                index = 2;
             case {-1}
                index = 3;
             case {-j}
                index = 4;
             otherwise
                error('*** GENERATE_DATA.M : Error in WQPSK generation! ***');
             end;
             d_array = signal_constellation(Data.Type, 1);
             Data.Symbol(pos) = d_array(index);
          end;
       end;
    case {'WDQPSK_Z120','WDQPSK_16QAM','WDQPSK_16QPSK','WDQPSK_C','WDQPSK_Boost','DQPSK_Boost'}
       %%% WARNING!!!
       error('*** GENERATE_DATA.M : Not Implemented Yet! ***');
       Data.Bit = binary(2*Data.Length);
       Data.Symbol = dqpsk(qpsk(Data.Bit));
       switch (Data.Block)
       case {'0 Symbol'}
       otherwise
          [TrainBlock, DataBlock] = data_block(Data.Block);
       end;
       for blk_cnt = 1 : size(TrainBlock,1)
          for pos = [TrainBlock(blk_cnt,1):TrainBlock(blk_cnt,2)]
             switch (Data.Symbol(pos))
             case {+1}
                index = 1;
             case {+j}
                index = 2;
             case {-1}
                index = 3;
             case {-j}
                index = 4;
             otherwise
                error('*** GENERATE_DATA.M : Error in WDQPSK generation! ***');
             end;
             d_array = signal_constellation(Data.Type, 1);
             Data.Symbol(pos) = d_array(index);
          end;
       end;
    case {'WQPSK_Boost2'}
       %%% WARNING!!!
       error('*** GENERATE_DATA.M : Not Implemented Yet! ***');
       Data.Bit = binary(2*Data.Length);
       Data.Symbol = dqpsk(qpsk(Data.Bit));
       switch (Data.Block)
       case {'0 Symbol'}
       otherwise
          [TrainBlock, DataBlock] = data_block(Data.Block);
       end;
       for blk_cnt = 1 : size(TrainBlock,1)
          for pos = [TrainBlock(blk_cnt,1):TrainBlock(blk_cnt,2)]
             switch (Data.Symbol(pos))
             case {+1}
                index = 1;
             case {+j}
                index = 2;
             case {-1}
                index = 3;
             case {-j}
                index = 4;
             otherwise
                error('*** GENERATE_DATA.M : Error in WDQPSK_Boost2 generation! ***');
             end;
             d_array = signal_constellation(Data.Type, 1);
             Data.Symbol(pos) = d_array(index);
          end;
          pos = TrainBlock(blk_cnt,2)+1;
          switch (Data.Symbol(pos))
          case {+1}
             index = 1;
          case {+j}
             index = 2;
          case {-1}
             index = 3;
          case {-j}
             index = 4;
          otherwise
             error('*** GENERATE_DATA.M : Error in WDQPSK_Boost2 generation! ***');
          end;
          d_array = signal_constellation(Data.Type, 2);
          Data.Symbol(pos) = d_array(index);
       end;
    case {'WDQPSK_Boost2'}
       %%% WARNING!!!
       error('*** GENERATE_DATA.M : Not Implemented Yet! ***');
       Data.Bit = binary(2*Data.Length);
       Data.Symbol = dqpsk(qpsk(Data.Bit));
       switch (Data.Block)
       case {'0 Symbol'}
       otherwise
          [TrainBlock, DataBlock] = data_block(Data.Block);
       end;
       for blk_cnt = 1 : size(TrainBlock,1)
          for pos = [TrainBlock(blk_cnt,1):TrainBlock(blk_cnt,2)]
             switch (Data.Symbol(pos))
             case {+1}
                index = 1;
             case {+j}
                index = 2;
             case {-1}
                index = 3;
             case {-j}
                index = 4;
             otherwise
                error('*** GENERATE_DATA.M : Error in WDQPSK_Boost2 generation! ***');
             end;
             d_array = signal_constellation(Data.Type, 1);
             Data.Symbol(pos) = d_array(index);
          end;
          pos = TrainBlock(blk_cnt,2)+1;
          switch (Data.Symbol(pos))
          case {+1}
             index = 1;
          case {+j}
             index = 2;
          case {-1}
             index = 3;
          case {-j}
             index = 4;
          otherwise
             error('*** GENERATE_DATA.M : Error in WDQPSK_Boost2 generation! ***');
          end;
          d_array = signal_constellation(Data.Type, 2);
          Data.Symbol(pos) = d_array(index);
       end;
    case {'WQPSK_Alt'}
       %%% WARNING!!!
       error('*** GENERATE_DATA.M : Not Implemented Yet! ***');
       Data.Bit = binary(2*Data.Length);
       Data.Symbol = qpsk(Data.Bit);
       for n = [1:2:Data.Length]
          switch (Data.Symbol(n))
          case {+1}
             index = 1;
          case {+j}
             index = 2;
          case {-1}
             index = 3;
          case {-j}
             index = 4;
          otherwise
             error('*** GENERATE_DATA.M : Error in WQPSK_A generation! ***');
          end;
          d_array = signal_constellation(Data.Type, n);
          Data.Symbol(n) = d_array(index);
       end;
    case {'WDQPSK_Alt'}
       %%% WARNING!!!
       error('*** GENERATE_DATA.M : Not Implemented Yet! ***');
       Data.Bit = binary(2*Data.Length);
       qpsk_symbol = qpsk(Data.Bit);
       for n = [1:2:Data.Length]
          Data.Symbol(n:n+1) = dqpsk(qpsk_symbol(n:n+1));
          switch (Data.Symbol(n))
          case {+1}
             index = 1;
          case {+j}
             index = 2;
          case {-1}
             index = 3;
          case {-j}
             index = 4;
          otherwise
             error('*** GENERATE_DATA.M : Error in WDQPSK_A generation! ***');
          end;
          d_array = signal_constellation(Data.Type, n);
          Data.Symbol(n) = d_array(index);
       end;
    case {'WDQPSK_DQPSK_64MRDPSK'}
       %%% WARNING!!!
       error('*** GENERATE_DATA.M : Not Implemented Yet! ***');
       switch (Data.Block)
       case {'0 Symbol'}
       otherwise
          [TrainBlock, DataBlock] = data_block(Data.Block);
       end;
       Data.Bit = [];
       Data.Symbol = [];
       for blk_cnt = 1 : size(TrainBlock,1)
          TN = TrainBlock(blk_cnt,2)-TrainBlock(blk_cnt,1)+1;
          DN = DataBlock(blk_cnt,2)-DataBlock(blk_cnt,1)+1;
          blk_bit = binary(2*TN+6*DN);
          t_blk = dqpsk(qpsk(blk_bit(1:2*TN)));
          d_blk = a64_mr_dpsk(blk_bit(2*TN+1:2*TN+6*DN));
          for pos = 1 : length(t_blk)
             switch (t_blk(pos))
             case {+1}
                index = 1;
             case {+j}
                index = 2;
             case {-1}
                index = 3;
             case {-j}
                index = 4;
             otherwise
                error('*** GENERATE_DATA.M : Error in 64MRDPSK generation! ***');
             end;
             d_array = signal_constellation(Data.Type, pos);
             t_blk(pos) = d_array(index);
          end;
          Data.Bit = [Data.Bit, blk_bit];
          Data.Symbol = [Data.Symbol, t_blk, d_blk];
       end;
    otherwise
       error('*** GENERATE_DATA.M : Unknown Data Type! ***');
    end;

    Fout.InSym(:,ii)= Data.Symbol;
end