function Fin = Receiver (ii,Fin, Fin_ch)
% Version 1.2
% Last Modified: May 24th, 2009
% Ali Tawfiq
% Multimedia Labratory, The Edward S. Rogers Sr. Dept of Electrical and
% Computer Engineering, University of Toronto

%**************************************************************************
%       ii      -> Itteration read at this instance
%       Fin     -> L: Length of signal
%               -> itt: number of iterations
%               -> Mod: Modulation Type (refer to 'Mapping.m' for allowed
%                        types)
%               -> M: M-ary, binary (2), quade (4) or 2^n
%               -> k: Number of bits per symbol
%               -> Block: String for block type (refer to 'Mapping.m')
%               -> BlockL: Length of blocks (refer to 'Mapping.m')
%               -> Numtx: Number of transmit antennas
%               -> Numrx: Number of receive antennas
%               -> SNR: a vector of SNR values
%               -> sBER: a temporary vector to store the computed BER
%               -> BER: a zero vector to store the computed BER
%               -> Out: a zero vector to store the final received signal
%                       to be be saved for reference
%               -> InSig: The input signal in bits
%               -> S: Numtx column vectors representing the split input
%                      signal
%               -> Saving: 'backup' for saving the data. 
%               -> InSym: Numtx column vectors of the modulated signal.
%               -> Px: Power of signal (1 for all psk, diff for QAM)
%               Filter Parameters:
%               -> r: Roll-off-factor Used in raised cosine filter
%               -> T: Bit period
%               -> dur:  Duration
%               -> FltrType: The type of filter generated. 
%               -> nsamp: oversampling rate
%               -> Delay: The delay experienced in the filter
%               -> fltr: The generated filter (To be applied in Transmit.m
%                        and Receiver.m)
%        Fin_ch -> Ch = array with channel coeficients as column vectors
%               -> R = array with received signals as the row vectors
%**************************************************************************    

%% ==================== Receiver =========================
%%========================================================
    for u= 1:length (Fin.SNR)
        % R is the received signal.
        R1= Fin_ch.R(u,:,1);% Received at antenna 1
        R2= Fin_ch.R(u,:,2);% Received at antenna 2

        % Filter received signal using square root raised cosine filter.
        % for R1
        rR1 = filter(Fin.fltr,1,R1).';
        rR1 = downsample(rR1,Fin.nsamp); % Downsample.
        rR1 = rR1((Fin.dur/Fin.T)+1:end-(Fin.dur/Fin.T)); % Account for delay.
        
        % Filter received signal using square root raised cosine filter.
        % for R2
        rR2 = filter(Fin.fltr,1,R2).';
        rR2 = downsample(rR2,Fin.nsamp); % Downsample.
        rR2 = rR2((Fin.dur/Fin.T)+1:end-(Fin.dur/Fin.T)); % Account for delay.
        
        %Create the two received signals
        r0 = rR1(1:2:length(rR1));
        r1 = rR1(2:2:length(rR1));
        r2 = rR2(1:2:length(rR2));
        r3 = rR2(2:2:length(rR2));
        
        % Downsample the channel coeficient
        rch0 = Fin_ch.Ch(1+(Fin.Delay/2):2*Fin.nsamp:length(Fin_ch.Ch)-3*(Fin.Delay/2),1); 
        rch1 = Fin_ch.Ch(1+(Fin.Delay/2):2*Fin.nsamp:length(Fin_ch.Ch)-3*(Fin.Delay/2),2); 
        rch2 = Fin_ch.Ch(1+(Fin.Delay/2):2*Fin.nsamp:length(Fin_ch.Ch)-3*(Fin.Delay/2),3); 
        rch3 = Fin_ch.Ch(1+(Fin.Delay/2):2*Fin.nsamp:length(Fin_ch.Ch)-3*(Fin.Delay/2),4); 
        
        % Combiner
        rS0 = r0.* conj(rch0) + rch1.* conj(r1) + r2.* conj(rch2) + rch3.* conj(r3);
        rS1 = r0.* conj(rch1) - rch0.* conj(r1) + r2.* conj(rch3) - rch2.* conj(r3);
        rS0 = rS0./(abs(rch0).^2 + abs(rch1).^2 + abs(rch2).^2 + abs(rch3).^2);
        rS1 = rS1./(abs(rch0).^2 + abs(rch1).^2 + abs(rch2).^2 + abs(rch3).^2);
        
%% ==================== Demapping ========================
%%========================================================
        % Combine symbole S1 and S2 together
        yrx= reshape([rS0,rS1].',1,[]);
        %Demodulate the signal
        %This specific simulation implements a BPSK demodulator. 
        Y= pskdemod(yrx, Fin.M);

%% =================== Conversion and reshaping ==========
%%========================================================
        % Convert integers to bits.
        Y= de2bi(Y,Fin.K,'left-msb'); 
        % Convert z from a matrix to a vector.
        Y = reshape((Y).',prod(size(Y)),1);
        
        [num,Fin.sBER(u)]= symerr(Fin.InSig(((ii-1)*Fin.L*Fin.K)+1 :ii*Fin.L*Fin.K),Y);
    end

    Fin.BER(ii, :)= Fin.sBER;
    Fin.Out=cat(1,Fin.Out,Y);
    
    